@@ -13,12 +13,10 @@ var CSSOM = {
 
 /**
  * @param {string} token
- * @param {Object} [options]
  */
-CSSOM.parse = function parse(token, options) {
+CSSOM.parse = function parse(token) {
 
-	options = options || {};
-	var i = options.startIndex || 0;
+	var i = 0;
 
 	/**
 	  "before-selector" or
@@ -30,10 +28,9 @@ CSSOM.parse = function parse(token, options) {
 	  "before-value" or
 	  "value"
 	*/
-	var state = options.state || "before-selector";
+	var state = "before-selector";
 
 	var index;
-	var j = i;
 	var buffer = "";
 
 	var SIGNIFICANT_WHITESPACE = {
@@ -47,11 +44,16 @@ CSSOM.parse = function parse(token, options) {
 
 	var styleSheet = new CSSOM.CSSStyleSheet;
 
-	// @type CSSStyleSheet|CSSMediaRule
+	// @type CSSStyleSheet|CSSMediaRule|CSSKeyframesRule
 	var currentScope = styleSheet;
-	
+
+	// @type CSSMediaRule|CSSKeyframesRule
+	var parentRule;
+
 	var selector, name, value, priority="", styleRule, mediaRule, importRule, keyframesRule, keyframeRule;
 
+	var atKeyframesRegExp = /@(-(?:\w+-)+)?keyframes/g;
+
 	for (var character; character = token.charAt(i); i++) {
 
 		switch (character) {
@@ -68,8 +70,7 @@ CSSOM.parse = function parse(token, options) {
 
 		// String
 		case '"':
-			j = i + 1;
-			index = token.indexOf('"', j) + 1;
+			index = token.indexOf('"', i + 1) + 1;
 			if (!index) {
 				throw '" is missing';
 			}
@@ -86,8 +87,7 @@ CSSOM.parse = function parse(token, options) {
 			break;
 
 		case "'":
-			j = i + 1;
-			index = token.indexOf("'", j) + 1;
+			index = token.indexOf("'", i + 1) + 1;
 			if (!index) {
 				throw "' is missing";
 			}
@@ -105,18 +105,18 @@ CSSOM.parse = function parse(token, options) {
 
 		// Comment
 		case "/":
-			if (token.charAt(i + 1) == "*") {
+			if (token.charAt(i + 1) === "*") {
 				i += 2;
 				index = token.indexOf("*/", i);
-				if (index == -1) {
-					throw SyntaxError("Missing */");
+				if (index === -1) {
+					throw new SyntaxError("Missing */");
 				} else {
 					i = index + 1;
 				}
 			} else {
 				buffer += character;
 			}
-			if (state == "importRule-begin") {
+			if (state === "importRule-begin") {
 				buffer += " ";
 				state = "importRule";
 			}
@@ -124,52 +124,60 @@ CSSOM.parse = function parse(token, options) {
 
 		// At-rule
 		case "@":
-			if (token.indexOf("@media", i) == i) {
+			if (token.indexOf("@media", i) === i) {
 				state = "atBlock";
 				mediaRule = new CSSOM.CSSMediaRule;
 				mediaRule.__starts = i;
 				i += "media".length;
 				buffer = "";
 				break;
-			} else if (token.indexOf("@import", i) == i) {
+			} else if (token.indexOf("@import", i) === i) {
 				state = "importRule-begin";
 				i += "import".length;
 				buffer += "@import";
 				break;
-			} else if (token.indexOf("@-webkit-keyframes", i) == i) {
-				state = "keyframesRule-begin";
-				keyframesRule = new CSSOM.CSSKeyframesRule;
-				keyframesRule.__starts = i;
-				i += "-webkit-keyframes".length;
-				buffer = "";
-				break;
-			} else if (state == "selector") {
-				state = "atRule";
+			} else {
+				atKeyframesRegExp.lastIndex = i;
+				var matchKeyframes = atKeyframesRegExp.exec(token);
+				if (matchKeyframes && matchKeyframes.index === i) {
+					state = "keyframesRule-begin";
+					keyframesRule = new CSSOM.CSSKeyframesRule;
+					keyframesRule.__starts = i;
+					keyframesRule._vendorPrefix = matchKeyframes[1]; // Will come out as undefined if no prefix was found
+					i += matchKeyframes[0].length - 1;
+					buffer = "";
+					break;
+				} else if (state == "selector") {
+					state = "atRule";
+				}
 			}
 			buffer += character;
 			break;
 
 		case "{":
-			if (state == "selector" || state == "atRule") {
-				styleRule.selectorText = buffer.trimRight();
+			if (state === "selector" || state === "atRule") {
+				styleRule.selectorText = buffer.trim();
 				styleRule.style.__starts = i;
 				buffer = "";
 				state = "before-name";
-			} else if (state == "atBlock") {
+			} else if (state === "atBlock") {
 				mediaRule.media.mediaText = buffer.trim();
-				mediaRule.parentRule = currentScope;
-				currentScope = mediaRule;
+				currentScope = parentRule = mediaRule;
+				mediaRule.parentStyleSheet = styleSheet;
 				buffer = "";
 				state = "before-selector";
-			} else if (state == "keyframesRule-begin") {
-				keyframesRule.name = buffer.trimRight();
-				keyframesRule.parentRule = currentScope;
-				currentScope = keyframesRule;
+			} else if (state === "keyframesRule-begin") {
+				keyframesRule.name = buffer.trim();
+				if (parentRule) {
+					keyframesRule.parentRule = parentRule;
+				}
+				keyframesRule.parentStyleSheet = styleSheet;
+				currentScope = parentRule = keyframesRule;
 				buffer = "";
 				state = "keyframeRule-begin";
-			} else if (state == "keyframeRule-begin") {
+			} else if (state === "keyframeRule-begin") {
 				styleRule = new CSSOM.CSSKeyframeRule;
-				styleRule.keyText = buffer.trimRight();
+				styleRule.keyText = buffer.trim();
 				styleRule.__starts = i;
 				buffer = "";
 				state = "before-name";
@@ -177,7 +185,7 @@ CSSOM.parse = function parse(token, options) {
 			break;
 
 		case ":":
-			if (state == "name") {
+			if (state === "name") {
 				name = buffer.trim();
 				buffer = "";
 				state = "before-value";
@@ -187,9 +195,9 @@ CSSOM.parse = function parse(token, options) {
 			break;
 
 		case '(':
-			if (state == 'value') {
+			if (state === 'value') {
 				index = token.indexOf(')', i + 1);
-				if (index == -1) {
+				if (index === -1) {
 					throw i + ': unclosed "("';
 				}
 				buffer += token.slice(i, index + 1);
@@ -200,7 +208,7 @@ CSSOM.parse = function parse(token, options) {
 			break;
 
 		case "!":
-			if (state == "value" && token.indexOf("!important", i) === i) {
+			if (state === "value" && token.indexOf("!important", i) === i) {
 				priority = "important";
 				i += "important".length;
 			} else {
@@ -222,9 +230,9 @@ CSSOM.parse = function parse(token, options) {
 					break;
 				case "importRule":
 					importRule = new CSSOM.CSSImportRule;
-					importRule.parentRule = currentScope;
+					importRule.parentStyleSheet = importRule.styleSheet.parentStyleSheet = styleSheet;
 					importRule.cssText = buffer + character;
-					currentScope.cssRules.push(importRule);
+					styleSheet.cssRules.push(importRule);
 					buffer = "";
 					state = "before-selector";
 					break;
@@ -242,10 +250,13 @@ CSSOM.parse = function parse(token, options) {
 				case "before-name":
 				case "name":
 					styleRule.__ends = i + 1;
-					styleRule.parentRule = currentScope;
+					if (parentRule) {
+						styleRule.parentRule = parentRule;
+					}
+					styleRule.parentStyleSheet = styleSheet;
 					currentScope.cssRules.push(styleRule);
 					buffer = "";
-					if (currentScope.constructor == CSSOM.CSSKeyframesRule) {
+					if (currentScope.constructor === CSSOM.CSSKeyframesRule) {
 						state = "keyframeRule-begin";
 					} else {
 						state = "before-selector";
@@ -255,13 +266,14 @@ CSSOM.parse = function parse(token, options) {
 				case "before-selector":
 				case "selector":
 					// End of media rule.
-					// Nesting rules aren't supported yet
-					if (!currentScope.parentRule) {
+					if (!parentRule) {
 						throw "unexpected }";
 					}
 					currentScope.__ends = i + 1;
-					currentScope.parentRule.cssRules.push(currentScope);
-					currentScope = currentScope.parentRule;
+					// Nesting rules arenâ€™t supported yet
+					styleSheet.cssRules.push(currentScope);
+					currentScope = styleSheet;
+					parentRule = null;
 					buffer = "";
 					state = "before-selector";
 					break;
